import sys

from .client import clear, feed_from_thredds, feed_from_directory

import logging
logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.WARN)
logger = logging.getLogger(__name__)


def create_parser():
    import argparse

    parser = argparse.ArgumentParser(
        prog="birdfeeder",
        usage='''birdfeeder [<options>] <command> [<args>]''',
        description="Feeds Solr with Datasets (NetCDF Format) from Thredds Catalogs and File System.",
        )
    parser.add_argument("--debug",
                        help="enable debug mode",
                        action="store_true")
    parser.add_argument("--service",
                        dest='service',
                        required=False,
                        type=type(''),
                        default='http://localhost:8983/solr/birdhouse',
                        help="Solr URL. Default: http://localhost:8983/solr/birdhouse",
                        action="store")
    parser.add_argument("--maxrecords",
                        dest='maxrecords',
                        required=False,
                        type=type(-1),
                        default=-1,
                        help="Maximum number of records to publish. Default: -1 (unlimited)",
                        action="store")
    parser.add_argument("--batch-size",
                        dest='batch_size',
                        required=False,
                        type=type(1),
                        default=50000,
                        help="Batch size of records to publish. Default: 50000",
                        action="store")
    subparsers = parser.add_subparsers(
            dest='command',
            title='command',
            description='List of available commands',
            help='Run "birdfeeder <command> -h" to get additional help.'
            )

    # clear command
    subparser = subparsers.add_parser(
        'clear',
        prog="birdfeeder clear",
        help="Clears the complete solr index. Use with caution!"
        )
     
    # from-thredds command
    subparser = subparsers.add_parser(
        'from-thredds',
        prog="birdfeeder from-thredds",
        help="Publish datasets from Thredds Catalog to Solr."
        )

    subparser.add_argument("--catalog-url",
                        dest='catalog_url',
                        required=True,
                        type=type(''),
                        help="Thredds Catalog URL",
                        action="store")
    subparser.add_argument("--depth",
                        dest='depth',
                        required=False,
                        type=type(0),
                        help="Depth level for Thredds catalog crawler",
                        action="store")

    # from-directory command
    subparser = subparsers.add_parser(
        'from-directory',
        prog="birdfeeder from-directory",
        help="Publish NetCDF files from directory to Solr."
        )

    subparser.add_argument("--start-dir",
                        dest='start_dir',
                        required=True,
                        type=type(''),
                        help="Start directory",
                        action="store")
   
    return parser


def execute(args):
    if args.debug:
        logger.setLevel(logging.DEBUG)
    if args.command == 'clear':
        clear(service=args.service)
    elif args.command == 'from-thredds':
        feed_from_thredds(service=args.service, catalog_url=args.catalog_url, depth=args.depth,
                          maxrecords=args.maxrecords, batch_size=args.batch_size)
    elif args.command == 'from-directory':
        feed_from_directory(service=args.service, start_dir=args.start_dir,
                            maxrecords=args.maxrecords,
                            batch_size=args.batch_size)
    logger.info('done.')

def main():
    import argcomplete

    logger.setLevel(logging.INFO)

    parser = create_parser()
    argcomplete.autocomplete(parser)

    args = parser.parse_args()
    execute(args)

if __name__ == '__main__':
    sys.exit(main())



import pysolr
import threddsclient
from . parser import ThreddsParser, NetCDFParser

import logging
logger = logging.getLogger(__name__)


def clear(service):
    solr = pysolr.Solr(service, timeout=10)
    logger.info("deletes all datasets from solr %s", service)
    solr.delete(q='*:*')
   

def feed_from_thredds(service, catalog_url, depth=1, maxrecords=-1, batch_size=50000):    
    logger.info("solr=%s, thredds catalog=%s", service, catalog_url)
    publish(service, parser=ThreddsParser(catalog_url, depth), maxrecords=maxrecords, batch_size=batch_size)
    
    
def feed_from_directory(service, start_dir, maxrecords=-1, batch_size=50000):
    logger.info("solr=%s, start dir=%s", service, start_dir)
    publish(service, parser=NetCDFParser(start_dir), maxrecords=maxrecords, batch_size=batch_size)


def publish(service, parser, maxrecords=-1, batch_size=50000):    
    solr = pysolr.Solr(service, timeout=10)

    records = []
    for metadata in parser.crawl():
        records.append(metadata)
        if len(records) >= batch_size:
            # publish if batch size is reached
            logger.info("publish %d records", len(records))
            solr.add(records)
            records = [] # reset records
        elif maxrecords >=0 and len(records) >= maxrecords:
            # stop publishing if max records reached
            break
    logger.info("publish %d records", len(records))
    solr.add(records)





# -*- coding: utf-8 -*-

from setuptools import find_packages
from setuptools import setup

version = '0.1.2'
description = 'Bird Feeder publishes Thredds metadata catalogs to a Solr index service with birdhouse schema.'
long_description = (
    open('README.rst').read() + '\n' +
    open('AUTHORS.rst').read() + '\n' +
    open('CHANGES.rst').read()
)

requires = [
    'argcomplete',
    'pysolr',
    'threddsclient',
    #'dateutil',
    'nose',
    ]

classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering :: Atmospheric Science',
        ]

setup(name='bird-feeder',
      version=version,
      description=description,
      long_description=long_description,
      classifiers=classifiers,
      keywords='thredds solr python netcdf birdhouse anaconda',
      author='Birdhouse Developers',
      author_email='',
      url='https://github.com/bird-house/bird-feeder',
      license = "Apache License v2.0",
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      test_suite='nose.collector',
      install_requires=requires,
      entry_points = {
          'console_scripts': [
              'birdfeeder=birdfeeder:main',
              ]}     
      ,
      )



import os
from netCDF4 import Dataset
import threddsclient
from dateutil import parser as dateparser

import logging
logger = logging.getLogger(__name__)


class Parser(object):
    """
    code is based on https://github.com/EarthSystemCoG/esgfpy-publish
    """
    def add_metadata(self, metadata, key, value):
        if not key in metadata:
            metadata[key] = [] 
        metadata[key].append(value)
    
    def crawl(self):
        raise NotImplemented


class ThreddsParser(Parser):
    def __init__(self, url, depth):
        Parser.__init__(self)
        self.url = url
        self.depth = depth
        self.cat = threddsclient.read_url(url)

    def parse(self, ds):
        metadata = dict(
            source=self.cat.url,
            title=ds.name,
            category='thredds',
            subject=self.cat.name,
            content_type=ds.content_type,
            last_modified=ds.modified,
            resourcename=ds.ID,
            url=ds.download_url(),
            opendap_url=ds.opendap_url(),
            wms_url=ds.wms_url(),
            catalog_url=ds.url)
        return metadata
    
    def crawl(self):
        for ds in threddsclient.crawl(self.url, depth=self.depth):
            yield self.parse(ds)


class NetCDFParser(Parser):
    SPATIAL_VARIABLES =  [
                'longitude', 'lon',
                'latitude', 'lat',
                'altitude', 'alt', 'level', 'height',
                'rotated_pole',
                'time']

        
    def __init__(self, start_dir):
        Parser.__init__(self)
        self.start_dir = start_dir

            
    def parse(self, filepath):
        filepath = os.path.abspath(filepath)
        logger.debug("parse %s", filepath)
        metadata = {}
        metadata['name'] = os.path.basename(filepath)
        metadata['url'] = 'file://' + filepath
        metadata['content_type'] = 'application/netcdf'
        metadata['resourcename'] = filepath

        try:
            ds = Dataset(filepath, 'r')

            # loop over global attributes
            for attname in ds.ncattrs():
                attvalue = getattr(ds, attname)
                if 'date' in attname.lower():
                    # must format dates in Solr format, if possible
                    try:
                        solr_dt = dateparser.parse(attvalue)
                        self.add_metadata(metadata, attname, solr_dt.strftime('%Y-%m-%dT%H:%M:%SZ') )
                    except:
                        pass # disregard this attribute
                else:
                    self.add_metadata(metadata, attname, attvalue)

            # loop over dimensions
            for key, dim in ds.dimensions.items():
                self.add_metadata(metadata, 'dimension', "%s:%s" % (key, len(dim)) )

            # loop over variable attributes
            for key, variable in ds.variables.items():
                if key.lower() in ds.dimensions:
                    # skip dimension variables
                    continue
                if '_bnds' in key.lower():
                    continue
                if key.lower() in self.SPATIAL_VARIABLES:
                    continue
                self.add_metadata(metadata, 'variable', key)
                self.add_metadata(metadata, 'variable_long_name', getattr(variable, 'long_name', None) )
                cf_standard_name = getattr(variable, 'standard_name', None)
                if cf_standard_name is not None:
                    self.add_metadata(metadata, 'cf_standard_name', getattr(variable, 'standard_name', None) )
                self.add_metadata(metadata, 'units', getattr(variable, 'units', None) )

        except Exception as e:
            logging.error(e)
        finally:
            try:
                ds.close()
            except:
                pass

        return metadata

    def map_fields(self, metadata):
        #logger.debug(metadata.keys())
        
        record = dict(
            source = self.start_dir,
            title = metadata.get('name'),
            category = "files",
            url = metadata.get('url'),
            content_type = metadata.get('content_type'),
            resourcename = metadata.get('resourcename'),
            variable = metadata.get('variable'),
            variable_long_name = metadata.get('variable_long_name'),
            cf_standard_name = metadata.get('cf_standard_name'),
            units = metadata.get('units'),
            comment = metadata.get('comments'),
            institute = metadata.get('institute_id'),
            experiment = metadata.get('experiment_id'),
            project = metadata.get('project_id'),
            model = metadata.get('model_id'),
            frequency = metadata.get('frequency'),
            creation_date = metadata.get('creation_date'),
            )
        return record

    def crawl(self):
        if not os.path.isdir(self.start_dir):
            raise Exception("Invalid start directory: %s", self.start_dir)
        
        logger.info('start directory = %s', self.start_dir)

        for directory, subdirs, files in os.walk(self.start_dir):
            # loop over files in this directory
            for filename in files:
                # ignore hidden files and thumbnails
                if not filename[0] == '.' and not 'thumbnail' in filename and not filename.endswith('.xml'):
                    filepath = os.path.join(directory, filename)
                    yield self.map_fields(self.parse(filepath))
        





